@@ -113,7 +113,7 @@ pub fn flip_neg_left<A: DProp, B: Prop>(f: Imply<Not<A>, B>) -> Imply<Not<B>, A>
     Rc::new(move |x| not::rev_double(g(x)))
 }
 
-/// `(¬b => a) => (¬a => b)`.
+/// `(a => ¬b) => (b => ¬a)`.
 pub fn flip_neg_right<A: Prop, B: Prop>(f: Imply<A, Not<B>>) -> Imply<B, Not<A>> {
     let g = imply::modus_tollens(f);
     Rc::new(move |x| g(not::double(x)))
@@ -139,6 +139,11 @@ pub fn in_right_arg<A: Prop, B: Prop, C: Prop>(f: Imply<A, B>, (g0, _): Eq<B, C>
     transitivity(f, g0)
 }
 
+/// `(a => c) ∧ (b => c)  =>  ((a ∧ b) => c)`.
+pub fn join<A: Prop, B: Prop, C: Prop>(f: Imply<A, C>, _: Imply<B, C>) -> Imply<And<A, B>, C> {
+    Rc::new(move |(a, _)| f.clone()(a))
+}
+
 /// `false => a`.
 pub fn absurd<A: Prop>() -> Imply<False, A> {
     Rc::new(|x| match x {})
